@@ -89,7 +89,6 @@ OC.L10N.register(
     "Automatically generate a birthday calendar" : "Luo syntymäpäiväkalenteri automaattisesti",
     "Birthday calendars will be generated by a background job." : "Syntymäpäiväkalenterit luodaan taustatyön toimesta.",
     "Hence they will not be available immediately after enabling but will show up after some time." : "Sen vuoksi ne eivät ole välittömästi saatavilla käyttöönoton jälkeen, vaan ne tulevat näkyviin pienellä viiveellä.",
-    "Send notifications for events" : "Lähetä ilmoitukset tapahtumista",
-    "Please make sure to properly set up the email settings above." : "Varmista, että määrität sähköpostiasetukset oikein yläpuolelle. "
+    "Send notifications for events" : "Lähetä ilmoitukset tapahtumista"
 },
 "nplurals=2; plural=(n != 1);");
