@@ -31,7 +31,9 @@ namespace OCA\Files\Controller;
 use OCA\Files\Activity\Helper;
 use OCP\AppFramework\Controller;
 use OCP\AppFramework\Http\ContentSecurityPolicy;
+use OCP\AppFramework\Http\NotFoundResponse;
 use OCP\AppFramework\Http\RedirectResponse;
+use OCP\AppFramework\Http\Response;
 use OCP\AppFramework\Http\TemplateResponse;
 use OCP\App\IAppManager;
 use OCP\Files\Folder;
@@ -128,6 +130,22 @@ class ViewController extends Controller {
 		return \OC_Helper::getStorageInfo('/', $dirInfo);
 	}
 
+	/**
+	 * @NoCSRFRequired
+	 * @NoAdminRequired
+	 *
+	 * @param string $fileid
+	 * @return TemplateResponse|RedirectResponse
+	 */
+	public function showFile(string $fileid = null): Response {
+		// This is the entry point from the `/f/{fileid}` URL which is hardcoded in the server.
+		try {
+			return $this->redirectToFile($fileid);
+		} catch (NotFoundException $e) {
+			return new RedirectResponse($this->urlGenerator->linkToRoute('files.view.index', ['fileNotFound' => true]));
+		}
+	}
+
 	/**
 	 * @NoCSRFRequired
 	 * @NoAdminRequired
@@ -140,7 +158,7 @@ class ViewController extends Controller {
 	public function index($dir = '', $view = '', $fileid = null, $fileNotFound = false) {
 		if ($fileid !== null) {
 			try {
-				return $this->showFile($fileid);
+				return $this->redirectToFile($fileid);
 			} catch (NotFoundException $e) {
 				return new RedirectResponse($this->urlGenerator->linkToRoute('files.view.index', ['fileNotFound' => true]));
 			}
@@ -251,8 +269,8 @@ class ViewController extends Controller {
 
 		$params                                = [];
 		$params['usedSpacePercent']            = (int) $storageInfo['relative'];
-		$params['owner']                       = $storageInfo['owner'];
-		$params['ownerDisplayName']            = $storageInfo['ownerDisplayName'];
+		$params['owner']                       = $storageInfo['owner'] ?? '';
+		$params['ownerDisplayName']            = $storageInfo['ownerDisplayName'] ?? '';
 		$params['isPublic']                    = false;
 		$params['allowShareWithLink']          = $this->config->getAppValue('core', 'shareapi_allow_links', 'yes');
 		$params['defaultFileSorting']          = $this->config->getUserValue($user, 'files', 'file_sorting', 'name');
@@ -285,7 +303,7 @@ class ViewController extends Controller {
 	 * @return RedirectResponse redirect response or not found response
 	 * @throws \OCP\Files\NotFoundException
 	 */
-	private function showFile($fileId) {
+	private function redirectToFile($fileId) {
 		$uid        = $this->userSession->getUser()->getUID();
 		$baseFolder = $this->rootFolder->getUserFolder($uid);
 		$files      = $baseFolder->getById($fileId);
