@@ -44,6 +44,7 @@ use Icewind\SMB\Exception\NotFoundException;
 use Icewind\SMB\Exception\TimedOutException;
 use Icewind\SMB\IFileInfo;
 use Icewind\SMB\Native\NativeServer;
+use Icewind\SMB\Options;
 use Icewind\SMB\ServerFactory;
 use Icewind\SMB\System;
 use Icewind\Streams\CallbackWrapper;
@@ -83,6 +84,9 @@ class SMB extends Common implements INotifyStorage {
 	/** @var ILogger */
 	protected $logger;
 
+	/** @var bool */
+	protected $showHidden;
+
 	public function __construct($params) {
 		if (!isset($params['host'])) {
 			throw new \Exception('Invalid configuration, no host provided');
@@ -103,7 +107,14 @@ class SMB extends Common implements INotifyStorage {
 			$this->logger = \OC::$server->getLogger();
 		}
 
-		$serverFactory = new ServerFactory();
+		$options = new Options();
+		if (isset($params['timeout'])) {
+			$timeout = (int)$params['timeout'];
+			if ($timeout > 0) {
+				$options->setTimeout($timeout);
+			}
+		}
+		$serverFactory = new ServerFactory($options);
 		$this->server = $serverFactory->createServer($params['host'], $auth);
 		$this->share = $this->server->getShare(trim($params['share'], '/'));
 
@@ -111,6 +122,8 @@ class SMB extends Common implements INotifyStorage {
 		$this->root = '/' . ltrim($this->root, '/');
 		$this->root = rtrim($this->root, '/') . '/';
 
+		$this->showHidden = isset($params['show_hidden']) && $params['show_hidden'];
+
 		$this->statCache = new CappedMemoryCache();
 		parent::__construct($params);
 	}
@@ -200,10 +213,13 @@ class SMB extends Common implements INotifyStorage {
 			}
 			return array_filter($files, function (IFileInfo $file) {
 				try {
-					if ($file->isHidden()) {
+					// the isHidden check is done before checking the config boolean to ensure that the metadata is always fetch
+					// so we trigger the below exceptions where applicable
+					$hide = $file->isHidden() && !$this->showHidden;
+					if ($hide) {
 						$this->logger->debug('hiding hidden file ' . $file->getName());
 					}
-					return !$file->isHidden();
+					return !$hide;
 				} catch (ForbiddenException $e) {
 					$this->logger->logException($e, ['level' => ILogger::DEBUG, 'message' => 'Hiding forbidden entry ' . $file->getName()]);
 					return false;
@@ -542,7 +558,7 @@ class SMB extends Common implements INotifyStorage {
 	public function isReadable($path) {
 		try {
 			$info = $this->getFileInfo($path);
-			return !$info->isHidden();
+			return $this->showHidden || !$info->isHidden();
 		} catch (NotFoundException $e) {
 			return false;
 		} catch (ForbiddenException $e) {
@@ -555,7 +571,7 @@ class SMB extends Common implements INotifyStorage {
 			$info = $this->getFileInfo($path);
 			// following windows behaviour for read-only folders: they can be written into
 			// (https://support.microsoft.com/en-us/kb/326549 - "cause" section)
-			return !$info->isHidden() && (!$info->isReadOnly() || $this->is_dir($path));
+			return ($this->showHidden || !$info->isHidden()) && (!$info->isReadOnly() || $this->is_dir($path));
 		} catch (NotFoundException $e) {
 			return false;
 		} catch (ForbiddenException $e) {
@@ -566,7 +582,7 @@ class SMB extends Common implements INotifyStorage {
 	public function isDeletable($path) {
 		try {
 			$info = $this->getFileInfo($path);
-			return !$info->isHidden() && !$info->isReadOnly();
+			return ($this->showHidden || !$info->isHidden()) && !$info->isReadOnly();
 		} catch (NotFoundException $e) {
 			return false;
 		} catch (ForbiddenException $e) {
