@@ -8,6 +8,7 @@ OC.L10N.register(
     "There was a problem saving your changes. Click to resume editing." : "Hebo un problema guardando tos cambéos. Calca pa reaniciar la edición.",
     "An error occurred!" : "Hebo un fallu!",
     "Edit" : "Editar",
+    "Preview" : "Previsualizar",
     "New text file" : "Ficheru nuevu de testu",
     "New text file.txt" : "Nuevu testu ficheru.txt",
     "You can not open a folder" : "Nun pues abrir una carpeta",
@@ -21,7 +22,6 @@ OC.L10N.register(
     "Insufficient permissions" : "Permisos insuficientes",
     "File path not supplied" : "Camín de ficheru non soportáu",
     "File mtime not supplied" : "Ficheru mtime non soportáu",
-    "Text editor" : "Editor de testos",
-    "Text file" : "Ficheru de testu"
+    "Text editor" : "Editor de testos"
 },
 "nplurals=2; plural=(n != 1);");
