@@ -1,7 +1,9 @@
 OC.L10N.register(
     "files_texteditor",
     {
+    "Saved" : "Lagra",
     "An error occurred!" : "Ein feil oppstod!",
+    "Edit" : "Endra",
     "This file is too big to be opened. Please download the file instead." : "Fila er for stor for å opnast. Ver venleg å laste ned fila i staden for.",
     "Cannot read the file." : "Kan ikkje lesa fila.",
     "Invalid file path supplied." : "Ugyldig stig oppgjett.",
