@@ -1,11 +1,19 @@
 OC.L10N.register(
     "firstrunwizard",
     {
+    "Copy to clipboard" : "Másolás vágólapra",
+    "Copied!" : "Másolva.",
+    "Not supported!" : "Nem támogatott.",
+    "Press ⌘-C to copy." : "A másoláshoz nyomj ⌘-C-t.",
+    "Press Ctrl-C to copy." : "A másoláshoz nyomj Ctrl-C-t.",
     "First run wizard" : "Első indíráskori varázsló",
     "Add your profile information! For example your email is needed to reset your password." : "Add meg a profil adataidat! Az e-mail címedre pédául jelszó visszaállításkor lehet szükség.",
     "Add your profile information! Set a profile picture and full name for easier recognition across all features." : "Add meg a profil információidat! Állíts be egy profilképet és teljes nevet az egyszerűbb azonosíthatóságért minden szolgáltatásban.",
     "Add your profile information! Set a full name for easier recognition across all features." : "Add meg a profil információidat! Állítsd be a teljes neved az egyszerűbb azonosíthatóságért minden szolgáltatásban.",
     "Add your profile information! Set a profile picture for easier recognition across all features." : "Add meg a profil információidat! Állítsd be egy profilképet az egyszerűbb azonosíthatóságért minden szolgáltatásban.",
+    "Schedule work & meetings, synced with all your devices." : "Ütemezett munkáját és találkozóit, szinkronizálva az összes eszközén.",
+    "Keep your colleagues and friends in one place without leaking their private info." : "Tartsa egy helyen kollégáit és barátait, anélkül hogy kiszivárogtatná a személyes adataikat.",
+    "Simple email app nicely integrated with Files, Contacts and Calendar." : "Egyszerű e-mail alkalmazás, amely szépen integrálódik a Fájlok, Névjegyek és Naptár alkalmazásba.",
     "About" : "Névjegy",
     "A welcome and introduction wizard for Nextcloud" : "Kezeld adataid és fájljaid ott, ahol te döntesz róluk",
     "Start using Nextcloud" : "A Nextcloud használatának kezdete",
@@ -50,11 +58,13 @@ OC.L10N.register(
     "100%% Open Source & community-focused" : "100%% Nyílt forráskodú és közösségi",
     "Learn more about %s" : "Tudj meg többet a %s-ról",
     "%s gives you access to your files wherever you are. Our easy to use desktop and mobile clients are available for all major platforms at zero cost." : "A %s hozzáférést biztosít a fájljaidhoz amikor csak akarod. A könnyen kezelhető desktop és mobil kliensünk ingyenesen elérhető az összes főbb platformon.",
-    "Setup sync clients using an <a href=\"%s\">app password</a>. That way you can make sure you are able to revoke access in case you lose that device." : "A szinkronizációs kliensek beállítása egy <a href=\"%s\">jelszó használatával</a>. Ezzel a módszerrel biztos lehetsz benne, hogy le tudod tiltani a hozzáférést ha elveszted azt az eszközt.",
     "Connect other apps to %s" : "Csatlakoztass más alkalmazást a %s-hoz",
-    "Besides the sync clients you can connect any other software that supports the WebDAV/CalDAV/CardDAV protocols to %s." : "A szinkronizációs kliensek használatával hozzácsatlakozhatsz bármilyen egyéb alkalmazáshoz ami támogatja a WebDAV/CalDAV/CardDAV protokolokat a %s-hoz.",
+    "Server address" : "Kiszolgálócím",
+    "Copy link" : "Hivatkozás másolása",
     "Previous" : "Előző",
     "Next" : "Következő",
+    "Setup sync clients using an <a href=\"%s\">app password</a>. That way you can make sure you are able to revoke access in case you lose that device." : "A szinkronizációs kliensek beállítása egy <a href=\"%s\">jelszó használatával</a>. Ezzel a módszerrel biztos lehetsz benne, hogy le tudod tiltani a hozzáférést ha elveszted azt az eszközt.",
+    "Besides the sync clients you can connect any other software that supports the WebDAV/CalDAV/CardDAV protocols to %s." : "A szinkronizációs kliensek használatával hozzácsatlakozhatsz bármilyen egyéb alkalmazáshoz ami támogatja a WebDAV/CalDAV/CardDAV protokolokat a %s-hoz.",
     "App recommendation: Nextcloud Talk" : "Alkalmazásjavaslat: Nextcloud Beszélgetés"
 },
 "nplurals=2; plural=(n != 1);");
