@@ -20,10 +20,9 @@ OC.L10N.register(
     "please install one of the following alternatives" : "prašome įdiegti vieną iš šių alternatyvų",
     "Album cannot be shown" : "Albumas negali būti parodytas",
     "No media files found" : "Nerasta duomenų peržiūrai",
-    "Upload pictures in the Files app to display them here" : "Norėdami matyti paveikslus čia, įkelkite juos į Failų programėlę",
+    "Upload pictures in the Files app to display them here" : "Norėdami čia matyti paveikslėlius, įkelkite juos į Failų programėlę",
     "Upload new files via drag and drop or by using the [+] button above" : "Įkelkite naujus failus, vilkdami juos čia arba pasinaudoję viršuje esančiu mygtuku [+]",
     "Configuration error" : "Konfigūracijos klaida",
-    "New" : "Naujas",
     "Hide Album" : "Slėpti albumą",
     "Could not hide album" : "Nepavyko paslėpti albumo",
     "Error loading slideshow template" : "Klaida, įkeliant skaidrių rodymo šabloną",
@@ -44,21 +43,21 @@ OC.L10N.register(
     "\"{name}\" is not an allowed filetype" : "\"{name}\" yra neleistinas failo tipas",
     "Upload" : "Įkelti",
     "Error" : "Klaida",
-    "Error while sharing" : "Klaida bandant pasidalinti",
+    "Error while sharing" : "Klaida bendrinant",
     "Error while unsharing" : "Klaida bandant nustoti dalintis",
     "Error while changing permissions" : "Klaida keičiant peržiūros leidimus",
     "Shared with you and the group {group} by {owner}" : "{owner} bendrina su jumis ir grupe {group}",
     "Shared with you by {owner}" : "{owner} bendrina su jumis",
-    "Share with users or groups …" : "Bendrinti su naudotojais ar grupėmis …",
-    "Share with users, groups or remote users …" : "Bendrinti su naudotojais, grupėmis ar naudotojais kituose serveriuose…",
+    "Share with users or groups …" : "Bendrinti su naudotojais ar grupėmis…",
+    "Share with users, groups or remote users …" : "Bendrinti su naudotojais, grupėmis ar nuotoliniais naudotojais…",
     "Share link" : "Bendrinimo nuoroda",
-    "The public link will expire no later than {days} days after it is created" : "Vieša nuoroda galios ne mažiau kaip {days} dienas nuo sukūrimo",
+    "The public link will expire no later than {days} days after it is created" : "Viešoji nuoroda galios ne mažiau kaip {days} dienas nuo sukūrimo",
     "Link" : "Nuoroda",
     "Password protect" : "Apsaugoti slaptažodžiu",
     "Password" : "Slaptažodis",
     "Choose a password for the public link" : "Pasirinkite viešos nuorodos slaptažodį",
     "Allow editing" : "Leisti redaguoti",
-    "Email link to person" : "Nusiųsti nuorodą paštu",
+    "Email link to person" : "Siųsti nuorodą el. paštu",
     "Send" : "Siųsti",
     "Set expiration date" : "Nustatykite galiojimo laiką",
     "Expiration" : "Galiojimo laikas",
@@ -87,14 +86,15 @@ OC.L10N.register(
     "Sending…" : "Siunčiama…",
     "Email sent" : "Laiškas išsiųstas",
     "Warning" : "Įspėjimas",
-    "shared by %s" : "pasidalino %s",
+    "shared by %s" : "bendrina %s",
     "Direct link" : "Tiesioginė nuoroda",
     "Add to your Nextcloud" : "Pridėti į savo Nextcloud",
-    "This share is password-protected" : "Turinys apsaugotas slaptažodžiu",
+    "Media gallery for Nextcloud" : "Medijos galerija, skirta Nextcloud",
+    "This share is password-protected" : "Šis viešinys yra apsaugotas slaptažodžiu",
     "The password is wrong. Try again." : "Slaptažodis neteisingas. Bandykite dar kartą.",
+    "Cancel upload" : "Atšaukti įkėlimą",
     "Sort by name" : "Rikiuoti pagal pavadinimą",
     "Sort by date" : "Rikiuoti pagal datą",
-    "Cancel upload" : "Atsisakyti įkėlimo",
     "Album information" : "Albumo informacija",
     "File list" : "Failų sąrašas",
     "Sorry, this file could not be found." : "Failas nerastas.",
@@ -110,15 +110,11 @@ OC.L10N.register(
     "a conflicting app was installed" : "nesuderinama programėlė buvo įdiegta",
     "a required component is missing or was disconnected" : "trūksta reikalingų komponentų arba jie buvo išjungti",
     "the filesystem is not readable" : "failų sistema nenuskaitoma",
-    "Sorry, this link doesn't seem to work anymore." : "Nuoroda yra neveiksni.",
+    "Sorry, this link doesn't seem to work anymore." : "Atleiskite, atrodo, kad ši nuoroda nebeveikia.",
     "the item was removed" : "elementas buvo pašalintas",
     "the link has expired" : "baigėsi nuorodos galiojimo laikas",
     "sharing is disabled" : "dalinimasis yra išjungtas",
     "For more information, please ask the person who has sent you this link." : "Dėl tikslesnės informacijos susisiekite su asmeniu atsiuntusiu nuorodą.",
-    "Picture view" : "Paveikslo rodymas",
-    "Gallery view" : "Galerijos rodinys",
-    "Share with people on other servers using their Federated Cloud ID username@example.com/cloud" : "Pasidalinti su naudotojais kituose serveriuose naudojantis Centralizuoto Serverio ID, pavyzdžiui: username@example.com/cloud",
-    "An error occured. Please try again" : "Klaida, prašau bandyti dar kartą.",
-    "Sending ..." : "Siunčiama..."
+    "Picture view" : "Paveikslėlių rodinys"
 },
 "nplurals=4; plural=(n % 10 == 1 && (n % 100 > 19 || n % 100 < 11) ? 0 : (n % 10 >= 2 && n % 10 <=9) && (n % 100 > 19 || n % 100 < 11) ? 1 : n % 1 != 0 ? 2: 3);");
