@@ -3,10 +3,15 @@
  * @copyright Copyright (c) 2016 Bjoern Schiessle <bjoern@schiessle.org>
  * @copyright Copyright (c) 2016 Lukas Reschke <lukas@statuscode.ch>
  *
+ * @author Arthur Schiwon <blizzz@arthur-schiwon.de>
  * @author Bjoern Schiessle <bjoern@schiessle.org>
+ * @author Daniel Calviño Sánchez <danxuliu@gmail.com>
+ * @author Jan-Christoph Borchardt <hey@jancborchardt.net>
+ * @author Joas Schilling <coding@schilljs.com>
  * @author Julius Haertl <jus@bitgrid.net>
+ * @author Julius Härtl <jus@bitgrid.net>
  * @author Lukas Reschke <lukas@statuscode.ch>
- * @author oparoz <owncloud@interfasys.ch>
+ * @author Robin Appelman <robin@icewind.nl>
  *
  * @license GNU AGPL version 3 or any later version
  *
@@ -27,8 +32,8 @@
 
 namespace OCA\Theming\Controller;
 
-use OC\Files\AppData\Factory;
 use OC\Template\SCSSCacher;
+use OCA\Theming\ImageManager;
 use OCA\Theming\ThemingDefaults;
 use OCP\AppFramework\Controller;
 use OCP\AppFramework\Http;
@@ -43,7 +48,6 @@ use OCP\Files\NotFoundException;
 use OCP\Files\NotPermittedException;
 use OCP\IConfig;
 use OCP\IL10N;
-use OCP\ILogger;
 use OCP\IRequest;
 use OCA\Theming\Util;
 use OCP\ITempManager;
@@ -62,8 +66,6 @@ class ThemingController extends Controller {
 	private $themingDefaults;
 	/** @var Util */
 	private $util;
-	/** @var ITimeFactory */
-	private $timeFactory;
 	/** @var IL10N */
 	private $l10n;
 	/** @var IConfig */
@@ -78,6 +80,8 @@ class ThemingController extends Controller {
 	private $urlGenerator;
 	/** @var IAppManager */
 	private $appManager;
+	/** @var ImageManager */
+	private $imageManager;
 
 	/**
 	 * ThemingController constructor.
@@ -87,13 +91,13 @@ class ThemingController extends Controller {
 	 * @param IConfig $config
 	 * @param ThemingDefaults $themingDefaults
 	 * @param Util $util
-	 * @param ITimeFactory $timeFactory
 	 * @param IL10N $l
 	 * @param ITempManager $tempManager
 	 * @param IAppData $appData
 	 * @param SCSSCacher $scssCacher
 	 * @param IURLGenerator $urlGenerator
 	 * @param IAppManager $appManager
+	 * @param ImageManager $imageManager
 	 */
 	public function __construct(
 		$appName,
@@ -101,19 +105,18 @@ class ThemingController extends Controller {
 		IConfig $config,
 		ThemingDefaults $themingDefaults,
 		Util $util,
-		ITimeFactory $timeFactory,
 		IL10N $l,
 		ITempManager $tempManager,
 		IAppData $appData,
 		SCSSCacher $scssCacher,
 		IURLGenerator $urlGenerator,
-		IAppManager $appManager
+		IAppManager $appManager,
+		ImageManager $imageManager
 	) {
 		parent::__construct($appName, $request);
 
 		$this->themingDefaults = $themingDefaults;
 		$this->util = $util;
-		$this->timeFactory = $timeFactory;
 		$this->l10n = $l;
 		$this->config = $config;
 		$this->tempManager = $tempManager;
@@ -121,13 +124,14 @@ class ThemingController extends Controller {
 		$this->scssCacher = $scssCacher;
 		$this->urlGenerator = $urlGenerator;
 		$this->appManager = $appManager;
+		$this->imageManager = $imageManager;
 	}
 
 	/**
 	 * @param string $setting
 	 * @param string $value
 	 * @return DataResponse
-	 * @internal param string $color
+	 * @throws NotPermittedException
 	 */
 	public function updateStylesheet($setting, $value) {
 		$value = trim($value);
@@ -152,6 +156,26 @@ class ThemingController extends Controller {
 					]);
 				}
 				break;
+			case 'imprintUrl':
+				if (strlen($value) > 500) {
+					return new DataResponse([
+						'data' => [
+							'message' => $this->l10n->t('The given legal notice address is too long'),
+						],
+						'status' => 'error'
+					]);
+				}
+				break;
+			case 'privacyUrl':
+				if (strlen($value) > 500) {
+					return new DataResponse([
+						'data' => [
+							'message' => $this->l10n->t('The given privacy policy address is too long'),
+						],
+						'status' => 'error'
+					]);
+				}
+				break;
 			case 'slogan':
 				if (strlen($value) > 500) {
 					return new DataResponse([
@@ -177,14 +201,14 @@ class ThemingController extends Controller {
 		$this->themingDefaults->set($setting, $value);
 
 		// reprocess server scss for preview
-		$cssCached = $this->scssCacher->process(\OC::$SERVERROOT, 'core/css/server.scss', 'core');
+		$cssCached = $this->scssCacher->process(\OC::$SERVERROOT, 'core/css/css-variables.scss', 'core');
 
 		return new DataResponse(
 			[
 				'data' =>
 					[
 						'message' => $this->l10n->t('Saved'),
-						'serverCssUrl' => $this->urlGenerator->linkTo('', $this->scssCacher->getCachedSCSS('core', '/core/css/server.scss'))
+						'serverCssUrl' => $this->urlGenerator->linkTo('', $this->scssCacher->getCachedSCSS('core', '/core/css/css-variables.scss'))
 					],
 				'status' => 'success'
 			]
@@ -192,33 +216,40 @@ class ThemingController extends Controller {
 	}
 
 	/**
-	 * Update the logos and background image
-	 *
 	 * @return DataResponse
+	 * @throws NotPermittedException
 	 */
-	public function updateLogo() {
-		$backgroundColor = $this->request->getParam('backgroundColor', false);
-		if($backgroundColor) {
-			$this->themingDefaults->set('backgroundMime', 'backgroundColor');
-			return new DataResponse(
-				[
-					'data' =>
-						[
-							'name' => 'backgroundColor',
-							'message' => $this->l10n->t('Saved')
-						],
-					'status' => 'success'
-				]
-			);
+	public function uploadImage(): DataResponse {
+		// logo / background
+		// new: favicon logo-header
+		//
+		$key = $this->request->getParam('key');
+		$image = $this->request->getUploadedFile('image');
+		$error = null;
+		$phpFileUploadErrors = [
+			UPLOAD_ERR_OK => $this->l10n->t('The file was uploaded'),
+			UPLOAD_ERR_INI_SIZE => $this->l10n->t('The uploaded file exceeds the upload_max_filesize directive in php.ini'),
+			UPLOAD_ERR_FORM_SIZE => $this->l10n->t('The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form'),
+			UPLOAD_ERR_PARTIAL => $this->l10n->t('The file was only partially uploaded'),
+			UPLOAD_ERR_NO_FILE => $this->l10n->t('No file was uploaded'),
+			UPLOAD_ERR_NO_TMP_DIR => $this->l10n->t('Missing a temporary folder'),
+			UPLOAD_ERR_CANT_WRITE => $this->l10n->t('Could not write file to disk'),
+			UPLOAD_ERR_EXTENSION => $this->l10n->t('A PHP extension stopped the file upload'),
+		];
+		if (empty($image)) {
+			$error = $this->l10n->t('No file uploaded');
 		}
-		$newLogo = $this->request->getUploadedFile('uploadlogo');
-		$newBackgroundLogo = $this->request->getUploadedFile('upload-login-background');
-		if (empty($newLogo) && empty($newBackgroundLogo)) {
+		if (!empty($image) && array_key_exists('error', $image) && $image['error'] !== UPLOAD_ERR_OK) {
+			$error = $phpFileUploadErrors[$image['error']];
+		}
+
+		if ($error !== null) {
 			return new DataResponse(
 				[
 					'data' => [
-						'message' => $this->l10n->t('No file uploaded')
-					]
+						'message' => $error
+					],
+					'status' => 'failure',
 				],
 				Http::STATUS_UNPROCESSABLE_ENTITY
 			);
@@ -231,83 +262,95 @@ class ThemingController extends Controller {
 			$folder = $this->appData->newFolder('images');
 		}
 
-		if (!empty($newLogo)) {
-			$target = $folder->newFile('logo');
-			$target->putContent(file_get_contents($newLogo['tmp_name'], 'r'));
-			$this->themingDefaults->set('logoMime', $newLogo['type']);
-			$name = $newLogo['name'];
-		}
-		if (!empty($newBackgroundLogo)) {
-			$target = $folder->newFile('background');
-			$image = @imagecreatefromstring(file_get_contents($newBackgroundLogo['tmp_name'], 'r'));
-			if ($image === false) {
-				return new DataResponse(
-					[
-						'data' => [
-							'message' => $this->l10n->t('Unsupported image type'),
-						],
-						'status' => 'failure',
-					],
-					Http::STATUS_UNPROCESSABLE_ENTITY
-				);
-			}
+		$this->imageManager->delete($key);
 
+		$target = $folder->newFile($key);
+		$supportedFormats = $this->getSupportedUploadImageFormats($key);
+		$detectedMimeType = mime_content_type($image['tmp_name']);
+		if (!in_array($image['type'], $supportedFormats) || !in_array($detectedMimeType, $supportedFormats)) {
+			return new DataResponse(
+				[
+					'data' => [
+						'message' => $this->l10n->t('Unsupported image type'),
+					],
+					'status' => 'failure',
+				],
+				Http::STATUS_UNPROCESSABLE_ENTITY
+			);
+		}
+
+		$resizeKeys = ['background'];
+		if (in_array($key, $resizeKeys, true)) {
 			// Optimize the image since some people may upload images that will be
 			// either to big or are not progressive rendering.
+			$newImage = @imagecreatefromstring(file_get_contents($image['tmp_name'], 'r'));
+
 			$tmpFile = $this->tempManager->getTemporaryFile();
-			if (function_exists('imagescale')) {
-				// FIXME: Once PHP 5.5.0 is a requirement the above check can be removed
-				// Workaround for https://bugs.php.net/bug.php?id=65171
-				$newHeight = imagesy($image) / (imagesx($image) / 1920);
-				$image = imagescale($image, 1920, $newHeight);
-			}
-			imageinterlace($image, 1);
-			imagejpeg($image, $tmpFile, 75);
-			imagedestroy($image);
+			$newWidth = imagesx($newImage) < 4096 ? imagesx($newImage) : 4096;
+			$newHeight = imagesy($newImage) / (imagesx($newImage) / $newWidth);
+			$outputImage = imagescale($newImage, $newWidth, $newHeight);
+
+			imageinterlace($outputImage, 1);
+			imagejpeg($outputImage, $tmpFile, 75);
+			imagedestroy($outputImage);
 
 			$target->putContent(file_get_contents($tmpFile, 'r'));
-			$this->themingDefaults->set('backgroundMime', $newBackgroundLogo['type']);
-			$name = $newBackgroundLogo['name'];
+		} else {
+			$target->putContent(file_get_contents($image['tmp_name'], 'r'));
 		}
+		$name = $image['name'];
+
+		$this->themingDefaults->set($key.'Mime', $image['type']);
+
+		$cssCached = $this->scssCacher->process(\OC::$SERVERROOT, 'core/css/css-variables.scss', 'core');
 
 		return new DataResponse(
 			[
 				'data' =>
 					[
 						'name' => $name,
-						'message' => $this->l10n->t('Saved')
+						'url' => $this->imageManager->getImageUrl($key),
+						'message' => $this->l10n->t('Saved'),
+						'serverCssUrl' => $this->urlGenerator->linkTo('', $this->scssCacher->getCachedSCSS('core', '/core/css/css-variables.scss'))
 					],
 				'status' => 'success'
 			]
 		);
 	}
 
+	/**
+	 * Returns a list of supported mime types for image uploads.
+	 * "favicon" images are only allowed to be SVG when imagemagick with SVG support is available.
+	 *
+	 * @param string $key The image key, e.g. "favicon"
+	 * @return array
+	 */
+	private function getSupportedUploadImageFormats(string $key): array {
+		$supportedFormats = ['image/jpeg', 'image/png', 'image/gif',];
+
+		if ($key !== 'favicon' || $this->imageManager->shouldReplaceIcons() === true) {
+			$supportedFormats[] = 'image/svg+xml';
+			$supportedFormats[] = 'image/svg';
+		}
+
+		return $supportedFormats;
+	}
+
 	/**
 	 * Revert setting to default value
 	 *
 	 * @param string $setting setting which should be reverted
 	 * @return DataResponse
+	 * @throws NotPermittedException
 	 */
-	public function undo($setting) {
+	public function undo(string $setting): DataResponse {
 		$value = $this->themingDefaults->undo($setting);
 		// reprocess server scss for preview
-		$cssCached = $this->scssCacher->process(\OC::$SERVERROOT, 'core/css/server.scss', 'core');
+		$cssCached = $this->scssCacher->process(\OC::$SERVERROOT, 'core/css/css-variables.scss', 'core');
 
-		if($setting === 'logoMime') {
-			try {
-				$file = $this->appData->getFolder('images')->getFile('logo');
-				$file->delete();
-			} catch (NotFoundException $e) {
-			} catch (NotPermittedException $e) {
-			}
-		}
-		if($setting === 'backgroundMime') {
-			try {
-				$file = $this->appData->getFolder('images')->getFile('background');
-				$file->delete();
-			} catch (NotFoundException $e) {
-			} catch (NotPermittedException $e) {
-			}
+		if (strpos($setting, 'Mime') !== -1) {
+			$imageKey = str_replace('Mime', '', $setting);
+			$this->imageManager->delete($imageKey);
 		}
 
 		return new DataResponse(
@@ -316,7 +359,7 @@ class ThemingController extends Controller {
 					[
 						'value' => $value,
 						'message' => $this->l10n->t('Saved'),
-						'serverCssUrl' => $this->urlGenerator->linkTo('', $this->scssCacher->getCachedSCSS('core', '/core/css/server.scss'))
+						'serverCssUrl' => $this->urlGenerator->linkTo('', $this->scssCacher->getCachedSCSS('core', '/core/css/css-variables.scss'))
 					],
 				'status' => 'success'
 			]
@@ -327,57 +370,42 @@ class ThemingController extends Controller {
 	 * @PublicPage
 	 * @NoCSRFRequired
 	 *
+	 * @param string $key
+	 * @param bool $useSvg
 	 * @return FileDisplayResponse|NotFoundResponse
+	 * @throws NotPermittedException
 	 */
-	public function getLogo() {
+	public function getImage(string $key, bool $useSvg = true) {
 		try {
-			/** @var File $file */
-			$file = $this->appData->getFolder('images')->getFile('logo');
+			$file = $this->imageManager->getImage($key, $useSvg);
 		} catch (NotFoundException $e) {
 			return new NotFoundResponse();
 		}
 
 		$response = new FileDisplayResponse($file);
+		$csp = new Http\ContentSecurityPolicy();
+		$csp->allowInlineStyle();
+		$response->setContentSecurityPolicy($csp);
 		$response->cacheFor(3600);
-		$expires = new \DateTime();
-		$expires->setTimestamp($this->timeFactory->getTime());
-		$expires->add(new \DateInterval('PT24H'));
-		$response->addHeader('Expires', $expires->format(\DateTime::RFC2822));
-		$response->addHeader('Pragma', 'cache');
-		$response->addHeader('Content-Type', $this->config->getAppValue($this->appName, 'logoMime', ''));
-		return $response;
-	}
-
-	/**
-	 * @PublicPage
-	 * @NoCSRFRequired
-	 *
-	 * @return FileDisplayResponse|NotFoundResponse
-	 */
-	public function getLoginBackground() {
-		try {
-			/** @var File $file */
-			$file = $this->appData->getFolder('images')->getFile('background');
-		} catch (NotFoundException $e) {
-			return new NotFoundResponse();
+		$response->addHeader('Content-Type', $this->config->getAppValue($this->appName, $key . 'Mime', ''));
+		$response->addHeader('Content-Disposition', 'attachment; filename="' . $key . '"');
+		if (!$useSvg) {
+			$response->addHeader('Content-Type', 'image/png');
+		} else {
+			$response->addHeader('Content-Type', $this->config->getAppValue($this->appName, $key . 'Mime', ''));
 		}
-
-		$response = new FileDisplayResponse($file);
-		$response->cacheFor(3600);
-		$expires = new \DateTime();
-		$expires->setTimestamp($this->timeFactory->getTime());
-		$expires->add(new \DateInterval('PT24H'));
-		$response->addHeader('Expires', $expires->format(\DateTime::RFC2822));
-		$response->addHeader('Pragma', 'cache');
-		$response->addHeader('Content-Type', $this->config->getAppValue($this->appName, 'backgroundMime', ''));
 		return $response;
 	}
 
 	/**
 	 * @NoCSRFRequired
 	 * @PublicPage
+	 * @NoSameSiteCookieRequired
 	 *
 	 * @return FileDisplayResponse|NotFoundResponse
+	 * @throws NotPermittedException
+	 * @throws \Exception
+	 * @throws \OCP\App\AppPathNotFoundException
 	 */
 	public function getStylesheet() {
 		$appPath = $this->appManager->getAppPath('theming');
@@ -395,11 +423,6 @@ class ThemingController extends Controller {
 			$cssFile = $this->scssCacher->getCachedCSS('theming', 'theming.css');
 			$response = new FileDisplayResponse($cssFile, Http::STATUS_OK, ['Content-Type' => 'text/css']);
 			$response->cacheFor(86400);
-			$expires = new \DateTime();
-			$expires->setTimestamp($this->timeFactory->getTime());
-			$expires->add(new \DateInterval('PT24H'));
-			$response->addHeader('Expires', $expires->format(\DateTime::RFC1123));
-			$response->addHeader('Pragma', 'cache');
 			return $response;
 		} catch (NotFoundException $e) {
 			return new NotFoundResponse();
@@ -409,6 +432,7 @@ class ThemingController extends Controller {
 	/**
 	 * @NoCSRFRequired
 	 * @PublicPage
+	 * @NoSameSiteCookieRequired
 	 *
 	 * @return DataDownloadResponse
 	 */
@@ -420,13 +444,46 @@ class ThemingController extends Controller {
 		url: ' . json_encode($this->themingDefaults->getBaseUrl()) . ',
 		slogan: ' . json_encode($this->themingDefaults->getSlogan()) . ',
 		color: ' . json_encode($this->themingDefaults->getColorPrimary()) . ',
+		imprintUrl: ' . json_encode($this->themingDefaults->getImprintUrl()) . ',
+		privacyUrl: ' . json_encode($this->themingDefaults->getPrivacyUrl()) . ',
 		inverted: ' . json_encode($this->util->invertTextColor($this->themingDefaults->getColorPrimary())) . ',
 		cacheBuster: ' . json_encode($cacheBusterValue) . '
 	};
 })();';
 		$response = new DataDownloadResponse($responseJS, 'javascript', 'text/javascript');
-		$response->addHeader('Expires', date(\DateTime::RFC2822, $this->timeFactory->getTime()));
-		$response->addHeader('Pragma', 'cache');
+		$response->cacheFor(3600);
+		return $response;
+	}
+
+	/**
+	 * @NoCSRFRequired
+	 * @PublicPage
+	 *
+	 * @return Http\JSONResponse
+	 */
+	public function getManifest($app) {
+		$cacheBusterValue = $this->config->getAppValue('theming', 'cachebuster', '0');
+		$responseJS = [
+			'name' => $this->themingDefaults->getName(),
+			'start_url' => $this->urlGenerator->getBaseUrl(),
+			'icons' =>
+				[
+					[
+						'src' => $this->urlGenerator->linkToRoute('theming.Icon.getTouchIcon',
+								['app' => $app]) . '?v=' . $cacheBusterValue,
+						'type'=> 'image/png',
+						'sizes'=> '128x128'
+					],
+					[
+						'src' => $this->urlGenerator->linkToRoute('theming.Icon.getFavicon',
+								['app' => $app]) . '?v=' . $cacheBusterValue,
+						'type' => 'image/svg+xml',
+						'sizes' => '16x16'
+					]
+				],
+			'display' => 'standalone'
+		];
+		$response = new Http\JSONResponse($responseJS);
 		$response->cacheFor(3600);
 		return $response;
 	}
