@@ -29,6 +29,12 @@ var OCdialogs = {
 	// dialog button types
 	YES_NO_BUTTONS:		70,
 	OK_BUTTONS:		71,
+
+	FILEPICKER_TYPE_CHOOSE: 1,
+	FILEPICKER_TYPE_MOVE: 2,
+	FILEPICKER_TYPE_COPY: 3,
+	FILEPICKER_TYPE_COPY_MOVE: 4,
+
 	// used to name each dialog
 	dialogsCounter: 0,
 	/**
@@ -76,6 +82,25 @@ var OCdialogs = {
 			modal
 		);
 	},
+	/**
+	* displays confirmation dialog
+	* @param text content of dialog
+	* @param title dialog title
+	* @param callback which will be triggered when user presses YES or NO
+	*        (true or false would be passed to callback respectively)
+	* @param modal make the dialog modal
+	*/
+	confirmHtml:function(text, title, callback, modal) {
+		return this.message(
+			text,
+			title,
+			'notice',
+			OCdialogs.YES_NO_BUTTONS,
+			callback,
+			modal,
+			true
+		);
+	},
 	/**
 	 * displays prompt dialog
 	 * @param text content of dialog
@@ -97,7 +122,7 @@ var OCdialogs = {
 				type       : 'notice'
 			});
 			var input = $('<input/>');
-			input.attr('type', password ? 'password' : 'text').attr('id', dialogName + '-input');
+			input.attr('type', password ? 'password' : 'text').attr('id', dialogName + '-input').attr('placeholder', name);
 			var label = $('<label/>').attr('for', dialogName + '-input').text(name + ': ');
 			$dlg.append(label);
 			$dlg.append(input);
@@ -105,6 +130,14 @@ var OCdialogs = {
 				modal = false;
 			}
 			$('body').append($dlg);
+
+			// wrap callback in _.once():
+			// only call callback once and not twice (button handler and close
+			// event) but call it for the close event, if ESC or the x is hit
+			if (callback !== undefined) {
+				callback = _.once(callback);
+			}
+
 			var buttonlist = [{
 					text : t('core', 'No'),
 					click: function () {
@@ -128,36 +161,85 @@ var OCdialogs = {
 			$(dialogId).ocdialog({
 				closeOnEscape: true,
 				modal        : modal,
-				buttons      : buttonlist
+				buttons      : buttonlist,
+				close        : function() {
+					// callback is already fired if Yes/No is clicked directly
+					if (callback !== undefined) {
+						callback(false, input.val());
+					}
+				}
 			});
+			input.focus();
 			OCdialogs.dialogsCounter++;
 		});
 	},
 	/**
 	 * show a file picker to pick a file from
+	 *
+	 * In order to pick several types of mime types they need to be passed as an
+	 * array of strings.
+	 *
+	 * When no mime type filter is given only files can be selected. In order to
+	 * be able to select both files and folders "['*', 'httpd/unix-directory']"
+	 * should be used instead.
+	 *
 	 * @param title dialog title
 	 * @param callback which will be triggered when user presses Choose
 	 * @param multiselect whether it should be possible to select multiple files
 	 * @param mimetypeFilter mimetype to filter by - directories will always be included
 	 * @param modal make the dialog modal
+	 * @param type Type of file picker : Choose, copy, move, copy and move
+	 * @param path path to the folder that the the file can be picket from
 	*/
-	filepicker:function(title, callback, multiselect, mimetypeFilter, modal) {
+	filepicker:function(title, callback, multiselect, mimetypeFilter, modal, type, path) {
 		var self = this;
+
+		this.filepicker.sortField = 'name';
+		this.filepicker.sortOrder = 'asc';
 		// avoid opening the picker twice
 		if (this.filepicker.loading) {
 			return;
 		}
+
+		if (type === undefined) {
+			type = this.FILEPICKER_TYPE_CHOOSE;
+		}
+
+		var emptyText = t('core', 'No files in here');
+		var newText = t('files', 'New folder');
+		if (type === this.FILEPICKER_TYPE_COPY || type === this.FILEPICKER_TYPE_MOVE || type === this.FILEPICKER_TYPE_COPY_MOVE) {
+			emptyText = t('core', 'No more subfolders in here');
+		}
+
 		this.filepicker.loading = true;
+		this.filepicker.filesClient = (OCA.Sharing && OCA.Sharing.PublicApp && OCA.Sharing.PublicApp.fileList)? OCA.Sharing.PublicApp.fileList.filesClient: OC.Files.getClient();
+
+		this.filelist = null;
+		path = path || '';
+
 		$.when(this._getFilePickerTemplate()).then(function($tmpl) {
 			self.filepicker.loading = false;
 			var dialogName = 'oc-dialog-filepicker-content';
 			if(self.$filePicker) {
 				self.$filePicker.ocdialog('close');
 			}
+
+			if (mimetypeFilter === undefined || mimetypeFilter === null) {
+				mimetypeFilter = [];
+			}
+			if (typeof(mimetypeFilter) === "string") {
+				mimetypeFilter = [mimetypeFilter];
+			}
+
 			self.$filePicker = $tmpl.octemplate({
 				dialog_name: dialogName,
-				title: title
-			}).data('path', '').data('multiselect', multiselect).data('mimetype', mimetypeFilter);
+				title: title,
+				emptytext: emptyText,
+				newtext: newText,
+				nameCol: t('core', 'Name'),
+				sizeCol: t('core', 'Size'),
+				modifiedCol: t('core', 'Modified')
+			}).data('path', path).data('multiselect', multiselect).data('mimetype', mimetypeFilter);
 
 			if (modal === undefined) {
 				modal = false;
@@ -165,52 +247,192 @@ var OCdialogs = {
 			if (multiselect === undefined) {
 				multiselect = false;
 			}
-			if (mimetypeFilter === undefined) {
-				mimetypeFilter = '';
+
+			// No grid for IE!
+			if (OC.Util.isIE()) {
+				self.$filePicker.find('#picker-view-toggle').remove();
+				self.$filePicker.find('#picker-filestable').removeClass('view-grid');
 			}
 
 			$('body').append(self.$filePicker);
 
-			self.$filePicker.ready(function() {
-				self.$filelist = self.$filePicker.find('.filelist');
-				self.$dirTree = self.$filePicker.find('.dirtree');
-				self.$dirTree.on('click', 'div:not(:last-child)', self, self._handleTreeListSelect.bind(self));
-				self.$filelist.on('click', 'li', function(event) {
-					self._handlePickerClick(event, $(this));
+			self.$showGridView = $('input#picker-showgridview');
+			self.$showGridView.on('change', _.bind(self._onGridviewChange, self));
+
+			if (!OC.Util.isIE()) {
+				self._getGridSettings();
+			}
+
+			var newButton = self.$filePicker.find('.actions.creatable .button-add');
+			if (type === self.FILEPICKER_TYPE_CHOOSE) {
+				newButton.hide();
+			}
+			newButton.on('focus', function() {
+				self.$filePicker.ocdialog('setEnterCallback', function() {
+					event.stopImmediatePropagation();
+					event.preventDefault();
+					newButton.click();
 				});
-				self._fillFilePicker('');
+			});
+			newButton.on('blur', function() {
+				self.$filePicker.ocdialog('unsetEnterCallback');
+			});
+
+			OC.registerMenu(newButton,self.$filePicker.find('.menu'),function () {
+				$input.focus();
+				self.$filePicker.ocdialog('setEnterCallback', function() {
+					event.stopImmediatePropagation();
+					event.preventDefault();
+					self.$form.submit();
+				});
+				var newName = $input.val();
+				var lastPos = newName.lastIndexOf('.');
+				if (lastPos === -1) {
+					lastPos = newName.length;
+				}
+				$input.selectRange(0, lastPos);
+			});
+			var $form = self.$filePicker.find('.filenameform');
+			var $input = $form.find('input[type=\'text\']');
+			var $submit = $form.find('input[type=\'submit\']');
+			$submit.on('click',function(event) {
+				event.stopImmediatePropagation();
+				event.preventDefault();
+				$form.submit();
+			});
+
+			var checkInput = function () {
+				var filename = $input.val();
+				try {
+					if (!Files.isFileNameValid(filename)) {
+						// Files.isFileNameValid(filename) throws an exception itself
+					} else if (self.filelist.find(function(file){return file.name === this;},filename)) {
+						throw t('files', '{newName} already exists', {newName: filename}, undefined, {
+							escape: false
+						});
+					} else {
+						return true;
+					}
+				} catch (error) {
+					$input.attr('title', error);
+					$input.tooltip({placement: 'right', trigger: 'manual', 'container': '.newFolderMenu'});
+					$input.tooltip('fixTitle');
+					$input.tooltip('show');
+					$input.addClass('error');
+				}
+				return false;
+			};
+
+			$form.on('submit', function(event) {
+				event.stopPropagation();
+				event.preventDefault();
+
+				if (checkInput()) { 
+					var newname = $input.val();
+					self.filepicker.filesClient.createDirectory(self.$filePicker.data('path') + '/' + newname).always(function (status) {
+						self._fillFilePicker(self.$filePicker.data('path') + '/' + newname );
+					});
+					OC.hideMenus();
+					self.$filePicker.ocdialog('unsetEnterCallback');
+					self.$filePicker.click();
+					$input.val(newText);
+				}
+			});
+			$input.keypress(function(event) {
+				if (event.keyCode === 13 || event.which === 13) {
+					event.stopImmediatePropagation();
+					event.preventDefault();
+					$form.submit();
+				}
+			});
+
+			self.$filePicker.ready(function() {
+				self.$fileListHeader = self.$filePicker.find('.filelist thead tr');
+				self.$filelist = self.$filePicker.find('.filelist tbody');
+				self.$filelistContainer = self.$filePicker.find('.filelist-container');
+				self.$dirTree = self.$filePicker.find('.dirtree');
+				self.$dirTree.on('click', 'div:not(:last-child)', self, function (event) {
+					self._handleTreeListSelect(event, type);
+				});
+				self.$filelist.on('click', 'tr', function(event) {
+					self._handlePickerClick(event, $(this), type);
+				});
+				self.$fileListHeader.on('click', 'a', function(event) {
+					var dir = self.$filePicker.data('path');
+					self.filepicker.sortField = $(event.currentTarget).data('sort');
+					self.filepicker.sortOrder = self.filepicker.sortOrder === 'asc' ? 'desc' : 'asc';
+					self._fillFilePicker(dir);
+				});
+				self._fillFilePicker(path);
 			});
 
 			// build buttons
-			var functionToCall = function() {
+			var functionToCall = function(returnType) {
 				if (callback !== undefined) {
 					var datapath;
 					if (multiselect === true) {
 						datapath = [];
-						self.$filelist.find('.filepicker_element_selected .filename').each(function(index, element) {
-							datapath.push(self.$filePicker.data('path') + '/' + $(element).text());
+						self.$filelist.find('tr.filepicker_element_selected').each(function(index, element) {
+							datapath.push(self.$filePicker.data('path') + '/' + $(element).data('entryname'));
 						});
 					} else {
 						datapath = self.$filePicker.data('path');
-						datapath += '/' + self.$filelist.find('.filepicker_element_selected .filename').text();
+						var selectedName = self.$filelist.find('tr.filepicker_element_selected').data('entryname');
+						if (selectedName) {
+							datapath += '/' + selectedName;
+						}
 					}
-					callback(datapath);
+					callback(datapath, returnType);
 					self.$filePicker.ocdialog('close');
 				}
 			};
-			var buttonlist = [{
-				text: t('core', 'Choose'),
-				click: functionToCall,
-				defaultButton: true
-			}];
+
+			var chooseCallback = function () {
+				functionToCall(OCdialogs.FILEPICKER_TYPE_CHOOSE);
+			};
+
+			var copyCallback = function () {
+				functionToCall(OCdialogs.FILEPICKER_TYPE_COPY);
+			};
+
+			var moveCallback = function () {
+				functionToCall(OCdialogs.FILEPICKER_TYPE_MOVE);
+			};
+
+			var buttonlist = [];
+			if (type === OCdialogs.FILEPICKER_TYPE_CHOOSE) {
+				buttonlist.push({
+					text: t('core', 'Choose'),
+					click: chooseCallback,
+					defaultButton: true
+				});
+			} else {
+				if (type === OCdialogs.FILEPICKER_TYPE_COPY || type === OCdialogs.FILEPICKER_TYPE_COPY_MOVE) {
+					buttonlist.push({
+						text: t('core', 'Copy'),
+						click: copyCallback,
+						defaultButton: false
+					});
+				}
+				if (type === OCdialogs.FILEPICKER_TYPE_MOVE || type === OCdialogs.FILEPICKER_TYPE_COPY_MOVE) {
+					buttonlist.push({
+						text: t('core', 'Move'),
+						click: moveCallback,
+						defaultButton: true
+					});
+				}
+			}
 
 			self.$filePicker.ocdialog({
 				closeOnEscape: true,
 				// max-width of 600
-				width: Math.min((4/5)*$(document).width(), 600),
-				height: 420,
+				width: 600,
+				height: 500,
 				modal: modal,
 				buttons: buttonlist,
+				style: {
+					buttons: 'aside',
+				},
 				close: function() {
 					try {
 						$(this).ocdialog('destroy').remove();
@@ -218,8 +440,15 @@ var OCdialogs = {
 					self.$filePicker = null;
 				}
 			});
-			if (!OC.Util.hasSVGSupport()) {
-				OC.Util.replaceSVG(self.$filePicker.parent());
+
+			// We can access primary class only from oc-dialog.
+			// Hence this is one of the approach to get the choose button.
+			var getOcDialog = self.$filePicker.closest('.oc-dialog');
+			var buttonEnableDisable = getOcDialog.find('.primary');
+			if (self.$filePicker.data('mimetype').indexOf("httpd/unix-directory") !== -1) {
+				buttonEnableDisable.prop("disabled", false);
+			} else {
+				buttonEnableDisable.prop("disabled", true);
 			}
 		})
 		.fail(function(status, error) {
@@ -235,7 +464,7 @@ var OCdialogs = {
 	 * Displays raw dialog
 	 * You better use a wrapper instead ...
 	*/
-	message:function(content, title, dialogType, buttons, callback, modal) {
+	message:function(content, title, dialogType, buttons, callback, modal, allowHtml) {
 		return $.when(this._getMessageTemplate()).then(function($tmpl) {
 			var dialogName = 'oc-dialog-' + OCdialogs.dialogsCounter + '-content';
 			var dialogId = '#' + dialogName;
@@ -244,7 +473,7 @@ var OCdialogs = {
 				title: title,
 				message: content,
 				type: dialogType
-			});
+			}, allowHtml ? {escapeFunction: ''} : {});
 			if (modal === undefined) {
 				modal = false;
 			}
@@ -280,7 +509,7 @@ var OCdialogs = {
 					}
 				};
 				buttonlist[0] = {
-					text: t('core', 'Ok'),
+					text: t('core', 'OK'),
 					click: functionToCall,
 					defaultButton: true
 				};
@@ -515,7 +744,7 @@ var OCdialogs = {
 						.prop('checked', true)
 						.prop('disabled', true);
 				$originalDiv.find('.message')
-					.text(t('core','read-only'))
+					.text(t('core','read-only'));
 			}
 		};
 		//var selection = controller.getSelection(data.originalFiles);
@@ -667,13 +896,37 @@ var OCdialogs = {
 		//}
 		return dialogDeferred.promise();
 	},
+	// get the gridview setting and set the input accordingly
+	_getGridSettings: function() {
+		var self = this;
+		$.get(OC.generateUrl('/apps/files/api/v1/showgridview'), function(response) {
+			self.$showGridView.get(0).checked = response.gridview;
+			self.$showGridView.next('#picker-view-toggle')
+				.removeClass('icon-toggle-filelist icon-toggle-pictures')
+				.addClass(response.gridview ? 'icon-toggle-filelist' : 'icon-toggle-pictures')
+			$('.list-container').toggleClass('view-grid', response.gridview);
+		});
+	},
+	_onGridviewChange: function() {
+		var show = this.$showGridView.is(':checked');
+		// only save state if user is logged in
+		if (OC.currentUser) {
+			$.post(OC.generateUrl('/apps/files/api/v1/showgridview'), {
+				show: show
+			});
+		}
+		this.$showGridView.next('#picker-view-toggle')
+			.removeClass('icon-toggle-filelist icon-toggle-pictures')
+			.addClass(show ? 'icon-toggle-filelist' : 'icon-toggle-pictures')
+		$('.list-container').toggleClass('view-grid', show);
+	},
 	_getFilePickerTemplate: function() {
 		var defer = $.Deferred();
 		if(!this.$filePickerTemplate) {
 			var self = this;
 			$.get(OC.filePath('core', 'templates', 'filepicker.html'), function(tmpl) {
 				self.$filePickerTemplate = $(tmpl);
-				self.$listTmpl = self.$filePickerTemplate.find('.filelist li:first-child').detach();
+				self.$listTmpl = self.$filePickerTemplate.find('.filelist tbody tr:first-child').detach();
 				defer.resolve(self.$filePickerTemplate);
 			})
 			.fail(function(jqXHR, textStatus, errorThrown) {
@@ -716,7 +969,7 @@ var OCdialogs = {
 		}
 		return defer.promise();
 	},
-	_getFileList: function(dir, mimeType) {
+	_getFileList: function(dir, mimeType) { //this is only used by the spreedme app atm
 		if (typeof(mimeType) === "string") {
 			mimeType = [mimeType];
 		}
@@ -734,56 +987,125 @@ var OCdialogs = {
 	 * fills the filepicker with files
 	*/
 	_fillFilePicker:function(dir) {
-		var dirs = [];
-		var others = [];
 		var self = this;
-		this.$filelist.empty().addClass('icon-loading');
+		this.$filelist.empty();
+		this.$filePicker.find('.emptycontent').hide();
+		this.$filelistContainer.addClass('icon-loading');
 		this.$filePicker.data('path', dir);
-		$.when(this._getFileList(dir, this.$filePicker.data('mimetype'))).then(function(response) {
+		var filter = this.$filePicker.data('mimetype');
+		if (typeof(filter) === "string") {
+			filter = [filter];
+		}
+		self.$fileListHeader.find('.sort-indicator').addClass('hidden').removeClass('icon-triangle-n').removeClass('icon-triangle-s');
+		self.$fileListHeader.find('[data-sort=' + self.filepicker.sortField + '] .sort-indicator').removeClass('hidden');
+		if (self.filepicker.sortOrder === 'asc') {
+			self.$fileListHeader.find('[data-sort=' + self.filepicker.sortField + '] .sort-indicator').addClass('icon-triangle-n');
+		} else {
+			self.$fileListHeader.find('[data-sort=' + self.filepicker.sortField + '] .sort-indicator').addClass('icon-triangle-s');
+		}
+		self.filepicker.filesClient.getFolderContents(dir).then(function(status, files) {
+			self.filelist = files;
+			if (filter && filter.length > 0 && filter.indexOf('*') === -1) {
+				files = files.filter(function (file) {
+					return file.type === 'dir' || filter.indexOf(file.mimetype) !== -1;
+				});
+			}
 
-			$.each(response.data.files, function(index, file) {
-				if (file.type === 'dir') {
-					dirs.push(file);
-				} else {
-					others.push(file);
+			var Comparators = {
+				name: function(fileInfo1, fileInfo2) {
+					if (fileInfo1.type === 'dir' && fileInfo2.type !== 'dir') {
+						return -1;
+					}
+					if (fileInfo1.type !== 'dir' && fileInfo2.type === 'dir') {
+						return 1;
+					}
+					return OC.Util.naturalSortCompare(fileInfo1.name, fileInfo2.name);
+				},
+				size: function(fileInfo1, fileInfo2) {
+					return fileInfo1.size - fileInfo2.size;
+				},
+				mtime: function(fileInfo1, fileInfo2) {
+					return fileInfo1.mtime - fileInfo2.mtime;
 				}
+			};
+			var comparator = Comparators[self.filepicker.sortField] || Comparators.name;
+			files = files.sort(function(file1, file2) {
+				var isFavorite = function(fileInfo) {
+					return fileInfo.tags && fileInfo.tags.indexOf(OC.TAG_FAVORITE) >= 0;
+				};
+
+				if (isFavorite(file1) && !isFavorite(file2)) {
+					return -1;
+				} else if (!isFavorite(file1) && isFavorite(file2)) {
+					return 1;
+				}
+
+				return self.filepicker.sortOrder === 'asc' ? comparator(file1, file2) : -comparator(file1, file2);
 			});
 
 			self._fillSlug();
-			var sorted = dirs.concat(others);
 
-			$.each(sorted, function(idx, entry) {
+			if (files.length === 0) {
+				self.$filePicker.find('.emptycontent').show();
+				self.$fileListHeader.hide();
+			} else {
+				self.$filePicker.find('.emptycontent').hide();
+				self.$fileListHeader.show();
+			}
+
+			$.each(files, function(idx, entry) {
 				entry.icon = OC.MimeType.getIconUrl(entry.mimetype);
-				var $li = self.$listTmpl.octemplate({
+				var simpleSize, sizeColor;
+				if (typeof(entry.size) !== 'undefined' && entry.size >= 0) {
+					simpleSize = humanFileSize(parseInt(entry.size, 10), true);
+					sizeColor = Math.round(160 - Math.pow((entry.size / (1024 * 1024)), 2));
+				} else {
+					simpleSize = t('files', 'Pending');
+					sizeColor = 80;
+				}
+
+				// split the filename in half if the size is bigger than 20 char
+				// for ellipsis
+				if (entry.name.length >= 10) {
+					// leave maximum 10 letters
+					var split = Math.min(Math.floor(entry.name.length / 2), 10)
+					var filename1 = entry.name.substr(0, entry.name.length - split)
+					var filename2 = entry.name.substr(entry.name.length - split)
+				} else {
+					var filename1 = entry.name
+					var filename2 = ''
+				}
+
+				var $row = self.$listTmpl.octemplate({
 					type: entry.type,
 					dir: dir,
 					filename: entry.name,
-					date: OC.Util.relativeModifiedDate(entry.mtime)
+					filename1: filename1,
+					filename2: filename2,
+					date: OC.Util.relativeModifiedDate(entry.mtime),
+					size: simpleSize,
+					sizeColor: sizeColor,
+					icon: entry.icon
 				});
 				if (entry.type === 'file') {
 					var urlSpec = {
 						file: dir + '/' + entry.name,
+						x: 100,
+						y: 100
 					};
-					$li.find('img').attr('src', OC.MimeType.getIconUrl(entry.mimetype));
 					var img = new Image();
 					var previewUrl = OC.generateUrl('/core/preview.png?') + $.param(urlSpec);
 					img.onload = function() {
 						if (img.width > 5) {
-							$li.find('img').attr('src', previewUrl);
+							$row.find('td.filename').attr('style', 'background-image:url(' + previewUrl + ')');
 						}
 					};
 					img.src = previewUrl;
 				}
-				else {
-					$li.find('img').attr('src', OC.Util.replaceSVGIcon(entry.icon));
-				}
-				self.$filelist.append($li);
+				self.$filelist.append($row);
 			});
 
-			self.$filelist.removeClass('icon-loading');
-			if (!OC.Util.hasSVGSupport()) {
-				OC.Util.replaceSVG(self.$filePicker.find('.dirtree'));
-			}
+			self.$filelistContainer.removeClass('icon-loading');
 		});
 	},
 	/**
@@ -816,22 +1138,65 @@ var OCdialogs = {
 	/**
 	 * handle selection made in the tree list
 	*/
-	_handleTreeListSelect:function(event) {
+	_handleTreeListSelect:function(event, type) {
 		var self = event.data;
-		var dir = $(event.target).parent().data('dir');
+		var dir = $(event.target).closest('.crumb').data('dir');
 		self._fillFilePicker(dir);
+		var getOcDialog = (event.target).closest('.oc-dialog');
+		var buttonEnableDisable = $('.primary', getOcDialog);
+		this._changeButtonsText(type, dir.split(/[/]+/).pop());
+		if (this.$filePicker.data('mimetype').indexOf("httpd/unix-directory") !== -1) {
+			buttonEnableDisable.prop("disabled", false);
+		} else {
+			buttonEnableDisable.prop("disabled", true);
+		}
 	},
 	/**
 	 * handle clicks made in the filepicker
 	*/
-	_handlePickerClick:function(event, $element) {
+	_handlePickerClick:function(event, $element, type) {
+		var getOcDialog = this.$filePicker.closest('.oc-dialog');
+		var buttonEnableDisable = getOcDialog.find('.primary');
 		if ($element.data('type') === 'file') {
 			if (this.$filePicker.data('multiselect') !== true || !event.ctrlKey) {
 				this.$filelist.find('.filepicker_element_selected').removeClass('filepicker_element_selected');
 			}
 			$element.toggleClass('filepicker_element_selected');
+			buttonEnableDisable.prop("disabled", false);
 		} else if ( $element.data('type') === 'dir' ) {
 			this._fillFilePicker(this.$filePicker.data('path') + '/' + $element.data('entryname'));
+			this._changeButtonsText(type, $element.data('entryname'));
+			if (this.$filePicker.data('mimetype').indexOf("httpd/unix-directory") !== -1) {
+				buttonEnableDisable.prop("disabled", false);
+			} else {
+				buttonEnableDisable.prop("disabled", true);
+			}
+		}
+	},
+
+	/**
+	 * Handle
+	 * @param type of action
+	 * @param dir on which to change buttons text
+	 * @private
+	 */
+	_changeButtonsText: function(type, dir) {
+		var copyText = dir === '' ? t('core', 'Copy') : t('core', 'Copy to {folder}', {folder: dir});
+		var moveText = dir === '' ? t('core', 'Move') : t('core', 'Move to {folder}', {folder: dir});
+		var buttons = $('.oc-dialog-buttonrow button');
+		switch (type) {
+			case this.FILEPICKER_TYPE_CHOOSE:
+				break;
+			case this.FILEPICKER_TYPE_COPY:
+				buttons.text(copyText);
+				break;
+			case this.FILEPICKER_TYPE_MOVE:
+				buttons.text(moveText);
+				break;
+			case this.FILEPICKER_TYPE_COPY_MOVE:
+				buttons.eq(0).text(copyText);
+				buttons.eq(1).text(moveText);
+				break;
 		}
 	}
 };
