@@ -281,7 +281,8 @@ class Manager implements IManager {
 
 		// Check if we actually have share permissions
 		if (!$share->getNode()->isShareable()) {
-			$message_t = $this->l->t('You are not allowed to share %s', [$share->getNode()->getPath()]);
+			$path = $userFolder->getRelativePath($share->getNode()->getPath());
+			$message_t = $this->l->t('You are not allowed to share %s', [$path]);
 			throw new GenericShareException($message_t, $message_t, 404);
 		}
 
@@ -323,7 +324,8 @@ class Manager implements IManager {
 
 		// Check that we do not share with more permissions than we have
 		if ($share->getPermissions() & ~$permissions) {
-			$message_t = $this->l->t('Can’t increase permissions of %s', [$share->getNode()->getPath()]);
+			$path = $userFolder->getRelativePath($share->getNode()->getPath());
+			$message_t = $this->l->t('Can’t increase permissions of %s', [$path]);
 			throw new GenericShareException($message_t, $message_t, 404);
 		}
 
